package com.rehome.jpahefengweather.service;


import com.rehome.jpahefengweather.entity.TyphoonInfo;
import com.rehome.jpahefengweather.entity.ZjsltStorm;

import java.util.List;

public interface ZjsltStormService {
    //根据id查询台风
    ZjsltStorm findByTfid(String tfid);
    //根据年份查询台风列表
    List<ZjsltStorm> findByYear(String year);
    //根据年份查询台风列表
    List<ZjsltStorm> getTyhoonActivity();
    /**
     * 保存台风数据
     */
    void saveZjsltStorm(ZjsltStorm zjsltStorm);

    TyphoonInfo findTyphoonInfoByTfid(String tfid);
    /**
     * 保存台风路径数据
     */
    void saveTyphoonInfo(TyphoonInfo typhoonInfo);

}
